/**
 * Enumeration of the job-splitting options.
 */
var JobSplitting;

(function (JobSplitting) {
  // Note that this is deliberately not a const enum. The TypeScript compiler inlines const enums, which means that the
  // generated declaration file is *required* for compiling into valid JavaScript. However, the declaration file may not
  // always be taken into consideration or even available. For example, the parcel bundler uses `transpileModule()` for
  // TypeScript assets:
  // https://github.com/parcel-bundler/parcel/blob/parcel-bundler%401.12.3/packages/core/parcel-bundler/src/assets/TypeScriptAsset.js#L46-L49
  // However, `transpileModule()` is just a simple transform function:
  // https://github.com/Microsoft/TypeScript/wiki/Using-the-Compiler-API#a-simple-transform-function
  // It does not look at any imports at all:
  // https://github.com/Microsoft/TypeScript/issues/5243

  /**
   * The job needs to be executed by a single machine en bloc (that is, with a single job fragment).
   */
  JobSplitting["NONE"] = "none";
  /**
   * The job needs to be executed by a single machine but it allows preemption; that is, its execution may be
   * interrupted by other jobs.
   */

  JobSplitting["PREEMPTION"] = "preemption";
  /**
   * The job can be executed by multiple machines, and it also allows preemption.
   */

  JobSplitting["MULTIPLE_MACHINES"] = "multi";
})(JobSplitting || (JobSplitting = {}));
/**
 * Returns whether the given value is a {@link SchedulingFailure}.
 */


function isSchedulingFailure(value) {
  return typeof value === 'string';
}

export { JobSplitting, isSchedulingFailure };
//# sourceMappingURL=api-types.js.map
