(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global.ProjectPlanningJs = {}));
}(this, function (exports) { 'use strict';

  function _classCallCheck(instance, Constructor) {
    if (!(instance instanceof Constructor)) {
      throw new TypeError("Cannot call a class as a function");
    }
  }

  function _defineProperties(target, props) {
    for (var i = 0; i < props.length; i++) {
      var descriptor = props[i];
      descriptor.enumerable = descriptor.enumerable || false;
      descriptor.configurable = true;
      if ("value" in descriptor) descriptor.writable = true;
      Object.defineProperty(target, descriptor.key, descriptor);
    }
  }

  function _createClass(Constructor, protoProps, staticProps) {
    if (protoProps) _defineProperties(Constructor.prototype, protoProps);
    if (staticProps) _defineProperties(Constructor, staticProps);
    return Constructor;
  }

  function _toConsumableArray(arr) {
    return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
  }

  function _arrayWithoutHoles(arr) {
    if (Array.isArray(arr)) {
      for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) arr2[i] = arr[i];

      return arr2;
    }
  }

  function _iterableToArray(iter) {
    if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
  }

  function _nonIterableSpread() {
    throw new TypeError("Invalid attempt to spread non-iterable instance");
  }

  /**
   * Enumeration of the job-splitting options.
   */

  (function (JobSplitting) {
    // Note that this is deliberately not a const enum. The TypeScript compiler inlines const enums, which means that the
    // generated declaration file is *required* for compiling into valid JavaScript. However, the declaration file may not
    // always be taken into consideration or even available. For example, the parcel bundler uses `transpileModule()` for
    // TypeScript assets:
    // https://github.com/parcel-bundler/parcel/blob/parcel-bundler%401.12.3/packages/core/parcel-bundler/src/assets/TypeScriptAsset.js#L46-L49
    // However, `transpileModule()` is just a simple transform function:
    // https://github.com/Microsoft/TypeScript/wiki/Using-the-Compiler-API#a-simple-transform-function
    // It does not look at any imports at all:
    // https://github.com/Microsoft/TypeScript/issues/5243

    /**
     * The job needs to be executed by a single machine en bloc (that is, with a single job fragment).
     */
    JobSplitting["NONE"] = "none";
    /**
     * The job needs to be executed by a single machine but it allows preemption; that is, its execution may be
     * interrupted by other jobs.
     */

    JobSplitting["PREEMPTION"] = "preemption";
    /**
     * The job can be executed by multiple machines, and it also allows preemption.
     */

    JobSplitting["MULTIPLE_MACHINES"] = "multi";
  })(exports.JobSplitting || (exports.JobSplitting = {}));
  /**
   * Returns whether the given value is a {@link SchedulingFailure}.
   */


  function isSchedulingFailure(value) {
    return typeof value === 'string';
  }

  /**
   * A classical min-heap.
   */
  function parentIdx(idx) {
    return Math.trunc((idx - 1) / 2);
  }

  function leftChildIdx(idx) {
    return 2 * idx + 1;
  }

  function rightChildIdx(idx) {
    return 2 * idx + 2;
  }

  function swap(array, leftIdx, rightIdx) {
    var temp = array[leftIdx];
    array[leftIdx] = array[rightIdx];
    array[rightIdx] = temp;
  }

  function heapify(array, comparatorFn, rootIdx) {
    // Precondition: indices leftChildIdx(rootIdx) and rightChildIdx(rootIdx) are roots of heaps
    var currentIdx = rootIdx;

    do {
      var minIdx = currentIdx;
      var l = leftChildIdx(currentIdx);

      if (l < array.length && comparatorFn(array[l], array[minIdx]) < 0) {
        minIdx = l;
      }

      var r = rightChildIdx(currentIdx);

      if (r < array.length && comparatorFn(array[r], array[minIdx]) < 0) {
        minIdx = r;
      }

      if (minIdx === currentIdx) {
        break;
      }

      swap(array, currentIdx, minIdx);
      currentIdx = minIdx;
    } while (true);
  }
  /**
   * A binary min-heap.
   *
   * @typeparam T the type of elements held in this min heap
   */


  var MinHeap =
  /*#__PURE__*/
  function () {
    function MinHeap(iterable, comparatorFn) {
      _classCallCheck(this, MinHeap);

      this.comparatorFn_ = comparatorFn;
      this.array_ = Array.from(iterable);

      for (var i = Math.trunc(this.array_.length / 2) - 1; i >= 0; --i) {
        heapify(this.array_, this.comparatorFn_, i);
      }
    }
    /**
     * Inserts the given element into this min-heap.
     *
     * The runtime of this operation is O(log n).
     *
     * @param element the element to add
     */


    _createClass(MinHeap, [{
      key: "add",
      value: function add(element) {
        this.array_.push(element);
        var currentIdx = this.array_.length - 1;

        do {
          var p = parentIdx(currentIdx);

          if (currentIdx <= 0 || this.comparatorFn_(this.array_[p], this.array_[currentIdx]) < 0) {
            break;
          }

          swap(this.array_, p, currentIdx);
          currentIdx = p;
        } while (true);
      }
      /**
       * Retrieves and removes the minimum element of this min-heap.
       *
       * The runtime of this operation is O(log n).
       *
       * @return the minimum element or undefined if the min-heap is empty
       */

    }, {
      key: "extractMin",
      value: function extractMin() {
        if (this.array_.length === 0) {
          return undefined;
        } else if (this.array_.length === 1) {
          return this.array_.pop();
        }

        var min = this.array_[0];
        this.array_[0] = this.array_.pop();
        heapify(this.array_, this.comparatorFn_, 0);
        return min;
      }
      /**
       * Returns whether this min-heap is empty.
       */

    }, {
      key: "isEmpty",
      value: function isEmpty() {
        return this.array_.length === 0;
      }
    }]);

    return MinHeap;
  }();

  var JOB_DEFAULTS = Object.freeze({
    deliveryTime: 0,
    splitting: exports.JobSplitting.PREEMPTION,
    dependencies: [],
    releaseTime: 0,
    preAssignment: -1
  });
  /**
   * Runs the list scheduling algorithm on the given problem instance and returns the result.
   *
   * See [the project page](https://github.com/fschopp/project-planning-js) for more information on the algorithm.
   *
   * @param instance the problem instance
   * @return solution or a human-readable failure description if the problem instance is invalid (for example, has a
   *     cyclic dependency graph)
   */

  function computeSchedule(instance) {
    var nonNegativeInteger = function nonNegativeInteger(number) {
      return number >= 0 && Number.isInteger(number);
    };

    var positiveIntegers = function positiveIntegers(numbers) {
      return numbers.filter(function (number) {
        return number <= 0 || !Number.isInteger(number);
      }).length === 0;
    };

    var undefinedOrNonNegativeInteger = function undefinedOrNonNegativeInteger(number) {
      return number === undefined || nonNegativeInteger(number);
    };

    var undefinedOrNonNegativeIntegers = function undefinedOrNonNegativeIntegers() {
      for (var _len = arguments.length, numbers = new Array(_len), _key = 0; _key < _len; _key++) {
        numbers[_key] = arguments[_key];
      }

      return numbers.filter(function (number) {
        return !undefinedOrNonNegativeInteger(number);
      }).length === 0;
    };

    var undefinedOrIntegerWithinZeroTo = function undefinedOrIntegerWithinZeroTo(upperBoundExcl, number) {
      return number === undefined || nonNegativeInteger(number) && number < upperBoundExcl;
    };

    var integersWithinZeroTo = function integersWithinZeroTo(upperBoundExcl, numbers) {
      return numbers.filter(function (number) {
        return !(number >= 0 && nonNegativeInteger(number) && number < upperBoundExcl);
      }).length === 0;
    };

    var numMachines = instance.machineSpeeds.length;
    var numJobs = instance.jobs.length;

    if (numMachines === 0) {
      return 'At least one machine is required to compute a schedule.';
    } else if (!positiveIntegers(instance.machineSpeeds) || instance.jobs.filter(function (job) {
      return !nonNegativeInteger(job.size) || !undefinedOrNonNegativeIntegers(job.deliveryTime, job.releaseTime) || !undefinedOrIntegerWithinZeroTo(numMachines, job.preAssignment) || job.dependencies !== undefined && !integersWithinZeroTo(numJobs, job.dependencies);
    }).length > 0 || !undefinedOrNonNegativeInteger(instance.minFragmentSize)) {
      return 'All job processing times and machine speeds need to be non-negative integers. ' + 'All job dependency and pre-assignment indices need to be within bounds.';
    }

    var jobs = instance.jobs.map(function (job) {
      return Object.assign({}, JOB_DEFAULTS, job);
    });
    var minFragmentSize = instance.minFragmentSize === undefined ? 0 : instance.minFragmentSize;
    return new ListScheduling(numMachines, instance.machineSpeeds, jobs, minFragmentSize).schedule;
  }
  /**
   * Internal class whose sole purpose is to store algorithm state, so it does not have to be inconveniently passed around
   * between functions.
   */

  var ListScheduling =
  /*#__PURE__*/
  function () {
    function ListScheduling(numMachines, machineSpeeds, jobs, minFragmentSize) {
      _classCallCheck(this, ListScheduling);

      this.numMachines_ = numMachines;
      this.machineSpeeds_ = machineSpeeds;
      this.jobs_ = jobs;
      this.minFragmentSize_ = minFragmentSize;
      this.gapsLists_ = this.machineSpeeds_.map(function (ignoredMachineSpeed) {
        return {
          head: {
            startTime: Number.MIN_SAFE_INTEGER,
            endTime: 0
          },
          tail: {
            head: {
              startTime: 0,
              endTime: Number.MAX_SAFE_INTEGER
            },
            tail: null
          }
        };
      });
      this.schedule = this.computeSchedule();
    }
    /**
     * Returns the next “event” for the given machine.
     *
     * This method has side-effects. It updates field {@link Machine.previousGapsList} and {@link Machine.gapsList} of the
     * given machine.
     */


    _createClass(ListScheduling, [{
      key: "adjustGaps",

      /**
       * Adjust the linked list of gaps to account for the newly scheduled job fragment.
       *
       * There are 4 cases to consider. The new job fragment either:
       * 1. Fills out the current gap completely. In this case, the current gap needs to be removed
       *    entirely.
       * 2. Aligns with the start of the current gap, but ends before it. In this case, the start of the current gap is
       *    changed to when the new job fragment ends.
       * 3. Does not start with the current gap, but aligns with the end of it. In this case, the end of the current gap is
       *    changed to when the job fragment starts.
       * 4. If the current gap aligns with neither start nor end of the current gap, a new gap is inserted before the
       *    current gap. It starts with the current gap and ends with the start of the job fragment. The start of the
       *    current gap is then changed to the end of the job fragment.
       *
       * @param machineState State of the machine that the job fragment has been scheduled on.
       * @param end end time of the job fragment
       * @param commitGaps if true, then commit the updated gaps to the state that is shared across scheduling of individual
       *     jobs
       */
      value: function adjustGaps(machineState, end, commitGaps) {
        // machineState also points to state that is shared across scheduling of individual jobs. This shared state we must
        // not modify if dryRun is true. We therefore cannot make modifications to the properties of
        // machineState.previousGapsList or machineState.gap.
        var start = machineState.currentFragmentStart;
        var currentGap = machineState.gapsList.head;

        if (commitGaps) {
          if (start === currentGap.startTime && end === currentGap.endTime) {
            machineState.previousGapsList.tail = machineState.gapsList.tail;
          } else if (start === currentGap.startTime) {
            currentGap.startTime = end;
          } else if (end === currentGap.endTime) {
            currentGap.endTime = start;
            machineState.previousGapsList = machineState.gapsList; // Note that machineState.gapsList will be updated below.
          } else {
            machineState.previousGapsList.tail = {
              head: {
                startTime: currentGap.startTime,
                endTime: start
              },
              tail: machineState.gapsList
            };
            currentGap.startTime = end;
          }
        }

        if (end === currentGap.endTime) {
          // Note that machineState.gapsList.tail === null would imply that the current gap is the last gap; that is,
          // currentGap.endTime === Number.MAX_SAFE_INTEGER. But we are also in the case where (end === currentGap.endTime).
          // Since we don't support schedules that long, it is safe to assert:
          machineState.gapsList = machineState.gapsList.tail;
        } else if (!commitGaps) {
          machineState.gapsList = {
            head: {
              startTime: end,
              endTime: currentGap.endTime
            },
            tail: machineState.gapsList.tail
          };
        } // Upon return, we will be "before" the gap represented by machineState.gapsList.head.


        machineState.currentFragmentStart = null;
      }
    }, {
      key: "createJobFragment",
      value: function createJobFragment(machineState, end, scheduledJob) {
        if (scheduledJob) {
          var start = machineState.currentFragmentStart;

          if (end - start > 0) {
            var jobFragment = {
              machine: machineState.index,
              start: start,
              end: end,
              isWaiting: false
            };
            scheduledJob.push(jobFragment);
          }
        }

        this.adjustGaps(machineState, end, scheduledJob !== undefined);
      }
      /**
       * Schedules processing of a job and returns its completion time (excluding delivery time).
       */

    }, {
      key: "scheduleJob",
      value: function scheduleJob(availableMachineIndices, size, isPreemptible, earliestStart, scheduledJob) {
        var _this = this;

        var minFragmentSize = isPreemptible ? Math.min(size, this.minFragmentSize_) : size;
        var currentSpeed = 0;
        var lastTimestamp = earliestStart;
        var remainingSize = size;
        var machines = availableMachineIndices.map(function (index, availableIdx) {
          var gapsList = _this.gapsLists_[index];
          return {
            index: index,
            availableIdx: availableIdx,
            speed: _this.machineSpeeds_[index],
            currentFragmentStart: null,
            previousGapsList: gapsList,
            gapsList: gapsList.tail
          };
        });

        var _loop = function _loop() {
          var eventTime = Number.MAX_SAFE_INTEGER;
          var machine = machines[0];
          availableMachineIndices.forEach(function (index, availableIdx) {
            var minWallClockTime = Math.ceil(minFragmentSize / machines[availableIdx].speed);
            var machineEventTime = ListScheduling.nextTimeStamp(minWallClockTime, earliestStart, machines[availableIdx]);

            if (machineEventTime < eventTime) {
              eventTime = machineEventTime;
              machine = machines[availableIdx];
            }
          });
          var isProjectedEndTime = false;
          var currentProjectedEndTime = Math.ceil(lastTimestamp + remainingSize / currentSpeed);

          if (currentProjectedEndTime < eventTime) {
            eventTime = currentProjectedEndTime;
            isProjectedEndTime = true;
          }

          remainingSize -= (eventTime - lastTimestamp) * currentSpeed;

          if (!isProjectedEndTime) {
            if (machine.currentFragmentStart === null) {
              var nextGap = machine.gapsList.head;
              machine.currentFragmentStart = Math.max(earliestStart, nextGap.startTime);
              remainingSize -= (eventTime - machine.currentFragmentStart) * machine.speed;
              currentSpeed += machine.speed;
            } else {
              _this.createJobFragment(machine, eventTime, scheduledJob);

              currentSpeed -= machine.speed;
            }
          }

          lastTimestamp = eventTime;
        };

        while (remainingSize > 0) {
          _loop();
        }

        availableMachineIndices.forEach(function (ignoredIndex, availableIdx) {
          if (machines[availableIdx].currentFragmentStart !== null) {
            _this.createJobFragment(machines[availableIdx], lastTimestamp, scheduledJob);
          }
        });
        return lastTimestamp;
      }
      /**
       * Adds the given delivery time on the given machine to the given job schedule, and returns the overall completion
       * time of the job.
       */

    }, {
      key: "allMachines",
      value: function allMachines() {
        return {
          length: this.machineSpeeds_.length,
          forEach: function forEach(callbackfn) {
            for (var i = 0; i < this.length; ++i) {
              callbackfn(i, i);
            }
          },
          map: function map(callbackfn) {
            var array = [];
            array.length = this.length;

            for (var i = 0; i < this.length; ++i) {
              array[i] = callbackfn(i, i);
            }

            return array;
          }
        };
      }
    }, {
      key: "computeSchedule",
      value: function computeSchedule() {
        var noDependencyNodes = [];
        var jobGraphNodes = this.jobs_.map(function (job, index) {
          return {
            idx: index,
            numDependencies: job.dependencies.length,
            dependents: []
          };
        });

        for (var i = 0; i < this.jobs_.length; ++i) {
          var job = this.jobs_[i];
          var jobGraphNode = jobGraphNodes[i];
          var _iteratorNormalCompletion = true;
          var _didIteratorError = false;
          var _iteratorError = undefined;

          try {
            for (var _iterator = job.dependencies[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
              var dependencyIdx = _step.value;
              jobGraphNodes[dependencyIdx].dependents.push(jobGraphNode);
            }
          } catch (err) {
            _didIteratorError = true;
            _iteratorError = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion && _iterator["return"] != null) {
                _iterator["return"]();
              }
            } finally {
              if (_didIteratorError) {
                throw _iteratorError;
              }
            }
          }

          if (job.dependencies.length === 0) {
            noDependencyNodes.push(jobGraphNode);
          }
        }

        var allMachines = this.allMachines();
        var newSchedule = this.jobs_.map(function (ignoredJob) {
          return [];
        });
        var finishTimes = this.jobs_.map(function (ignoredJob) {
          return -1;
        });
        var numScheduledJobs = 0;
        var noDependencyNodesHeap = new MinHeap(noDependencyNodes, function (left, right) {
          return left.idx - right.idx;
        });

        while (!noDependencyNodesHeap.isEmpty()) {
          var _jobGraphNode = noDependencyNodesHeap.extractMin();

          var _job = this.jobs_[_jobGraphNode.idx];
          var isPreemptible = _job.splitting !== exports.JobSplitting.NONE;

          var earliestStartTime = _job.dependencies.reduce(function (previousEarliestStartTime, dependencyIdx) {
            var dependencyFinishTime = finishTimes[dependencyIdx];
            return Math.max(previousEarliestStartTime, dependencyFinishTime);
          }, _job.releaseTime);

          var availableMachines = allMachines;
          var deliveryMachineIdx = void 0;

          if (_job.splitting === exports.JobSplitting.MULTIPLE_MACHINES) {
            deliveryMachineIdx = _job.preAssignment >= 0 ? _job.preAssignment : 0;
          } else if (_job.preAssignment >= 0) {
            availableMachines = ListScheduling.singleMachine(_job.preAssignment);
            deliveryMachineIdx = _job.preAssignment;
          } else {
            var maxCompletionTime = Number.MAX_SAFE_INTEGER;

            for (var _i = 0; _i < this.numMachines_; ++_i) {
              var currentAvailableMachines = ListScheduling.singleMachine(_i);
              var currentCompletionTime = this.scheduleJob(currentAvailableMachines, _job.size, isPreemptible, earliestStartTime);

              if (currentCompletionTime < maxCompletionTime) {
                availableMachines = currentAvailableMachines;
                deliveryMachineIdx = _i;
                maxCompletionTime = currentCompletionTime;
              }
            }
          }

          var completionTime = this.scheduleJob(availableMachines, _job.size, isPreemptible, earliestStartTime, newSchedule[_jobGraphNode.idx]);
          finishTimes[_jobGraphNode.idx] = ListScheduling.scheduleDeliveryTime(deliveryMachineIdx, completionTime, _job.deliveryTime, newSchedule[_jobGraphNode.idx]);
          ++numScheduledJobs;
          var _iteratorNormalCompletion2 = true;
          var _didIteratorError2 = false;
          var _iteratorError2 = undefined;

          try {
            for (var _iterator2 = _jobGraphNode.dependents[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
              var dependent = _step2.value;
              --dependent.numDependencies;

              if (dependent.numDependencies === 0) {
                noDependencyNodesHeap.add(dependent);
              }
            }
          } catch (err) {
            _didIteratorError2 = true;
            _iteratorError2 = err;
          } finally {
            try {
              if (!_iteratorNormalCompletion2 && _iterator2["return"] != null) {
                _iterator2["return"]();
              }
            } finally {
              if (_didIteratorError2) {
                throw _iteratorError2;
              }
            }
          }
        }

        return numScheduledJobs !== this.jobs_.length ? 'Detected a cycle in the dependency graph.' : newSchedule;
      }
    }], [{
      key: "nextTimeStamp",
      value: function nextTimeStamp(minWallClockTime, earliestStart, machine) {
        var currentGap = machine.gapsList.head;

        if (machine.currentFragmentStart === null) {
          while (true) {
            var timeStamp = Math.max(earliestStart, currentGap.startTime) + minWallClockTime;

            if (timeStamp <= currentGap.endTime) {
              return timeStamp;
            }

            machine.previousGapsList = machine.gapsList;
            machine.gapsList = machine.gapsList.tail;
            currentGap = machine.gapsList.head;
          }
        } else {
          return currentGap.endTime;
        }
      }
    }, {
      key: "scheduleDeliveryTime",
      value: function scheduleDeliveryTime(machineIdx, processingCompletionTime, deliveryTime, scheduledJob) {
        var completionTime = processingCompletionTime;

        if (deliveryTime > 0) {
          completionTime += deliveryTime;
          var deliveryJobFragment = {
            machine: machineIdx,
            start: processingCompletionTime,
            end: completionTime,
            isWaiting: true
          };
          scheduledJob.push(deliveryJobFragment);
        }

        return completionTime;
      }
    }, {
      key: "singleMachine",
      value: function singleMachine(singleMachineIdx) {
        return {
          length: 1,
          forEach: function forEach(callbackfn) {
            callbackfn(singleMachineIdx, 0);
          },
          map: function map(callbackfn) {
            return [callbackfn(singleMachineIdx, 0)];
          }
        };
      }
    }]);

    return ListScheduling;
  }();

  /**
   * Internal message interface between the worker and the master.
   *
   * None of this is public API.
   */

  /**
   * Type guard that returns whether the given value is a {@link DedicatedWorkerGlobalScope}.
   */
  function isDedicatedWorkerGlobalScope(value) {
    return typeof DedicatedWorkerGlobalScope === 'function' && value instanceof DedicatedWorkerGlobalScope;
  }
  var workerFactory = {};

  if (isDedicatedWorkerGlobalScope(self)) {
    var onMessageFromMaster = function onMessageFromMaster(event) {
      var computeScheduleArgs = event.data;
      var result = computeSchedule.apply(void 0, _toConsumableArray(computeScheduleArgs));
      dedicatedWorkerGlobalScope.postMessage(result);
      dedicatedWorkerGlobalScope.close();
    };

    var dedicatedWorkerGlobalScope = self;
    dedicatedWorkerGlobalScope.onmessage = onMessageFromMaster;
  }

  function isHtmlScriptElement(scriptElement) {
    return 'src' in scriptElement;
  }

  var THIS_SCRIPT_URL = document && document.currentScript && isHtmlScriptElement(document.currentScript) && document.currentScript.src ? document.currentScript.src : undefined;

  workerFactory.createWorker = function () {
    // Notes:
    // 1. The expectation is that the current script is loaded through a <script> element with a context that sets
    //    document.currentScript. This excludes newer contexts such as running as a module script or running in a shadow
    //    tree. https://html.spec.whatwg.org/multipage/dom.html#dom-document-currentscript
    //    If you are seeing this error, either change the <script> element to a more traditional context, or use the ES6
    //    module distribution from dist/es6 and use a bundler for your web app.
    // 2. Current browsers do not support {type: 'module'} as `options` argument yet.
    return THIS_SCRIPT_URL === undefined ? 'Failed to create web worker because the URL of the current script could not be determined.' : new Worker(THIS_SCRIPT_URL);
  };

  /**
   * Runs (in a separate thread) the list scheduling algorithm on the given problem instance and returns the result
   * asynchronously.
   *
   * See [the project page](https://github.com/fschopp/project-planning-js) for more information on the algorithm.
   *
   * @param args argument list that will be passed on to {@link computeSchedule}() unaltered
   * @return promise that will be resolved with the solution or rejected with a {@link SchedulingFailure} containing a
   *     human-readable failure description if the problem instance is invalid (for example, has a cyclic dependency
   *     graph)
   */

  function computeScheduleAsync() {
    var worker = workerFactory.createWorker();

    if (isSchedulingFailure(worker)) {
      var failure = worker;
      return Promise.reject(failure);
    }
    // Hence all worker callbacks are in place before we send it the "go" message below.

    var promise = new Promise(function (resolve, reject) {
      worker.onmessage = function (event) {
        var result = event.data;

        if (isSchedulingFailure(result)) {
          reject(result);
        } else {
          resolve(result);
        }
      };

      worker.onerror = function (event) {
        worker.terminate();
        var failure = 'A runtime error occurred in source file ' + "".concat(event.filename, " (line ").concat(event.lineno, ":").concat(event.colno, "):\n").concat(event.message); // In theory, the worker could still cause an error after it has sent its last message. However, the ES6
        // specification says:
        // "Attempting to resolve or reject a resolved promise has no effect."
        // https://www.ecma-international.org/ecma-262/6.0/#sec-promise-objects
        // Hence, there is no race condition. On the other hand, we don't want to swallow the problem. Therefore, we
        // add an assertion:

        reject(failure);
      };
    });

    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    worker.postMessage(args);
    return promise;
  }

  exports.computeSchedule = computeSchedule;
  exports.computeScheduleAsync = computeScheduleAsync;
  exports.isSchedulingFailure = isSchedulingFailure;

  Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=index.js.map
